<?php    

if(!defined('ParticleTraffic_PATH')) die;

class ParticleTrafficDisplay{

	public $siteIdNumber=0;

	public static function loadAndDisplay($siteIdNumber=0,$noLoginRequired=0){
		$statsDisplay=new ParticleTrafficDisplay($siteIdNumber);
		if( !ParticalStatsAuthentication::isLoggedIn() && !$noLoginRequired ){ 
			if(!ParticalStatsAuthentication::login($_REQUEST['user'],$_REQUEST['pass']))
				return $statsDisplay->loginBox();
		}
		
		$statsDisplay->loadStatData();
		return $statsDisplay->displayStatsHTML();
	}

	public function __construct($siteIdNumber=0){
		$this->siteIdNumber=intval($siteIdNumber);
		$this->db = ParticleTrafficDB::get_instance(); 
	}	

	//for displaying the traffic analysis interface
	public function printHeaderFiles( $siteIdNumber=0 ){
		echo '<link href="'.ParticleTraffic_URL_DIR.'css/ParticleTraffic.structure.css" rel="stylesheet" type="text/css" />'."\n";
		ParticleTrafficDisplay::printDisplayJavascript($siteIdNumber);
	}
	
	public function printDisplayJavascript( $siteIdNumber=0 ){
		echo '<script src="'.ParticleTraffic_URL_DIR.'ParticleTraffic.services.php?mode=display_js&site='.intval($siteIdNumber).'" type="text/javascript"></script> '."\n";
	}
	
	public function loginBox(){
		ob_start(); 
		include(ParticleTraffic_PATH.'views/login_box.php');
		$html=ob_get_contents();
		ob_end_clean();
		return $html;
	}

	public function loadStatData(){ 
		$this->browserData=$this->getBrowserStats();
		$this->platformData=$this->getPlatformStats();
		$this->screenData=$this->getResolutionStats();
		$this->flashData=$this->getFlashStats();
		$this->referrerData['top']=$this->getTopReferrers();
		$this->referrerData['recent']=$this->getRecentReferrers();
		$this->searchesData['top']=$this->getTopSearches();
		$this->searchesData['recent']=$this->getRecentSearches();
		$this->pagesData['top']=$this->getTopPages();
		$this->pagesData['recent']=$this->getRecentPages();		
		$this->visitData=$this->getVisitData(); 
		$this->visitPaths=$this->getVisitPaths();
		$this->generateGraphXML($this->visitData,'year');
		$this->generateGraphXML($this->visitData,'day');
		$this->generateGraphXML($this->visitData,'week'); 
	}
	
	public function displayStatsHTML(){
		ob_start(); 
		include(ParticleTraffic_PATH.'views/main_template.php');
		$html=ob_get_contents();
		ob_end_clean();
		return $html;
	}

	public function getVisitData(){		
		//get cached month data
		$sql='SELECT * FROM '.ParticleTrafficTracker::$cache_table.' ';
		if(intval($this->siteIdNumber)) $sql.=' WHERE book_id='.intval($this->siteIdNumber).' '; 		
		$cachedMonthResults=$this->db->get_array($sql);
		
		//get recent hits data
		$sql='SELECT tv.traffic_visit_id, th.page_url, th.page_title, th.created FROM '.ParticleTrafficTracker::$hits_table.' AS th, '.ParticleTrafficTracker::$visits_table.' AS tv WHERE ';
		if(intval($this->siteIdNumber)) $sql.='  tv.book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' tv.traffic_visit_id=th.traffic_visits_id ORDER BY th.created desc'; 
		$results=$this->db->get_array($sql); 
		$visitData=array();
		$visit_ids=array();
		$visitData['months']=array();
		foreach($results as $result){
			$hitTime=$result['created'];
			$visitData['hours'][date('Y_n_z_G',$hitTime)]['pageviews']++;
			$visitData['days'][date('Y_n_z',$hitTime)]['pageviews']++;
			//$visitData['weeks'][date('Y_W',$hitTime)]['pageviews']++;
			$visitData['months'][date('Y_n',$hitTime)]['pageviews']++;
			$visitData['year'][date('Y',$hitTime)]['pageviews']++;
			//is unique?
			if(in_array($result['traffic_visit_id'],$visit_ids)) continue;
			$visitData['hours'][date('Y_n_z_G',$hitTime)]['visits']++;
			$visitData['days'][date('Y_n_z',$hitTime)]['visits']++;
			//$visitData['weeks'][date('Y_W',$hitTime)]['visits']++;
			$visitData['months'][date('Y_n',$hitTime)]['visits']++;
			$visitData['year'][date('Y',$hitTime)]['visits']++;
			$visit_ids[]=$result['traffic_visit_id'];
		}
		
		//cache older month data
		foreach($visitData['months'] as $dateKey=>$monthData){
			if( $dateKey==date('Y_n') || $dateKey==date('Y_n',(time()-(3600*24*40))) ) continue;
			$dateParts=explode('_',$dateKey);
			$month=intval($dateParts[1]);
			$year=intval($dateParts[0]);			
			$where='datetime > '.mktime(0,0,0,$month,0,$year).' AND datetime < '.mktime(0,0,0,$month+1,0,$year).' AND book_id='.intval($this->siteIdNumber);
			$cachedMonthVOs=ParticleTrafficCachedMonth::retrieveList($where,1,'datetime DESC');
			$datetime=mktime(1,1,1,$month,1,$year);
			if(!count($cachedMonthVOs)){ 
				$monthVO = new ParticleTrafficCachedMonth();
			}else{
				$monthVO=$cachedMonthVOs[0];			
			}
			$monthVO->set_book_id( intval($this->siteIdNumber) );
			$monthVO->set_datetime( $datetime );
			$monthVO->set_visits( $monthData['pageviews'] );
			$monthVO->set_pageviews( $monthData['visits'] );
			$monthVO->save();			
		}
		
		//cleanup old hits - over 62 days old
		if(intval($this->siteIdNumber)){
			$deleteAgeCutoff=(date('U')-(3600*24*62) );
			$sql='DELETE th.* FROM '.ParticleTrafficTracker::$hits_table.' as th, '.ParticleTrafficTracker::$visits_table.' as tv WHERE tv.traffic_visit_id=th.traffic_visits_id AND tv.book_id='.intval($this->siteIdNumber).' AND tv.created<'.$deleteAgeCutoff; 
			$this->db->query($sql);
			$sql='DELETE FROM '.ParticleTrafficTracker::$visits_table.' WHERE book_id='.intval($this->siteIdNumber).' AND created<'.$deleteAgeCutoff; 
			$this->db->query($sql);
		}
		
		//add in cached months
		foreach($cachedMonthResults as $result){ 
			$monthCacheDate=$result['datetime'];
			$dateKey=date('Y_n',$monthCacheDate);
			if( $dateKey==date('Y_n') || $dateKey==date('Y_n',(time()-(3600*24*40))) ) continue;
			$visitData['months'][$dateKey]['pageviews']=$result['pageviews'];
			$visitData['months'][$dateKey]['visits']=$result['visits']; 
		}
		
		return $visitData;
	}


	public static function cleanBrowserName($name=''){
		return ucwords( str_replace( array('msie'), array('Internet Explorer'),$name));
	}

	public function getBrowserStats(){
		$sql='SELECT browser, browser_version, count(*) as count FROM '.ParticleTrafficTracker::$visits_table.'  ';
		if(intval($this->siteIdNumber)) $sql.=' WHERE book_id='.intval($this->siteIdNumber).' ';
		$sql.=' GROUP BY browser, browser_version ORDER BY created desc LIMIT 1000'; 
		$results=$this->db->get_array($sql);
		$familyData=array();
		$totalCount=0; 
		foreach($results as $result){
			if($result['browser']=='unrecognized') continue; 
			$totalCount=$totalCount+intval($result['count']);
		}
		if(!$totalCount) return array();
		foreach($results as $result){
			if($result['browser']=='unrecognized') continue; 
			$browser=$result['browser'];
			$familyData[$browser]['browser']=$browser;
			$familyData[$browser]['count'] = intval($familyData[$browser]['count'])+intval($result['count']);
			$percent = (intval($familyData[$browser]['count']) / $totalCount * 100);
			$familyData[$browser]['usage'] = $percent;
			$version=intval($result['browser_version']);
			if($version>0){
				if(!$familyData[$browser]['validVersionsCount']) $familyData[$browser]['validVersionsCount']=0;
				$familyData[$browser]['validVersionsCount']=intval($familyData[$browser]['validVersionsCount'])+intval($result['count']);
				$familyData[$browser]['versions'][$version]['version']=$version;
				$familyData[$browser]['versions'][$version]['count']=intval($familyData[$browser]['versions'][$version]['count'])+intval($result['count']);
				//$percent = floor(intval($familyData[$browser]['versions'][$version]['count']) / intval($familyData[$browser]['validVersionsCount']) * 100);
				//$familyData[$browser]['versions'][$version]['usage'] = $percent;
			}
		}  
		foreach($familyData as $browser=>$data){
			$vc=$data['validVersionsCount'];
			if($data['versions']) foreach($data['versions'] as $versionNumber=>$versionData){
				$percent = (intval($versionData['count']) / intval($data['validVersionsCount']) * 100);
				$familyData[$data['browser']]['versions'][$versionData['version']]['usage'] = $percent;
			}
		}
		usort($familyData, 'sortByTrafficUsage'); 
		$familyData=array_reverse($familyData);
		$familyData=array_slice($familyData,0,15);		
		return $familyData;
	}
	
	public function getPlatformStats(){
		$sql='SELECT platform, count(*) as count FROM '.ParticleTrafficTracker::$visits_table.'  ';
		if(intval($this->siteIdNumber)) $sql.=' WHERE book_id='.intval($this->siteIdNumber).' ';
		$sql.=' GROUP BY platform ORDER BY created desc LIMIT 1000'; 
		$results=$this->db->get_array($sql);
		$familyData=array();
		$totalCount=0; 
		foreach($results as $result){
			if($result['platform']=='unrecognized') continue; 
			$totalCount=$totalCount+intval($result['count']);
		}
		if(!$totalCount) return array();
		foreach($results as $result){
			if($result['platform']=='unrecognized') continue; 
			$id=$result['platform'];
			$familyData[$id]['count'] = intval($familyData[$id]['count'])+intval($result['count']);
			$percent = floor(intval($familyData[$id]['count']) / $totalCount * 100);
			$familyData[$id]['usage'] = $percent;
			$familyData[$id]['platform'] = $result['platform'];
		}
		usort($familyData, 'sortByTrafficUsage'); 
		$familyData=array_reverse($familyData);
		$familyData=array_slice($familyData,0,15);		
		return $familyData;
	}
	
	public function getFlashStats(){
		$sql='SELECT flash_version, count(*) as count FROM '.ParticleTrafficTracker::$visits_table.' WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' resolution!="" GROUP BY flash_version ORDER BY created desc LIMIT 1000'; 
		$results=$this->db->get_array($sql);
		$familyData=array();
		$totalCount=0; 
		foreach($results as $result){
			$totalCount=$totalCount+intval($result['count']);
		}
		if(!$totalCount) return array();
		foreach($results as $result){ 
			$id='version'.$result['flash_version'].'.0';
			$familyData[$id]['version']=$result['flash_version'].'.0';
			$familyData[$id]['count'] = intval($familyData[$id]['count'])+intval($result['count']);
			$percent = floor(intval($familyData[$id]['count']) / $totalCount * 100);
			$familyData[$id]['usage'] = $percent;
		}   	
		usort($familyData, 'sortByTrafficUsage'); 
		$familyData=array_reverse($familyData);
		$familyData=array_slice($familyData,0,15);		
		return $familyData;
	}	
	
	public function getResolutionStats(){
		$sql='SELECT resolution, count(*) as count FROM '.ParticleTrafficTracker::$visits_table.' ';
		if(intval($this->siteIdNumber)) $sql.=' WHERE book_id='.intval($this->siteIdNumber).' ';
		$sql.=' GROUP BY resolution ORDER BY resolution LIMIT 1000'; 
		$results=$this->db->get_array($sql);
		$familyData=array();
		$totalCount=0; 
		foreach($results as $result){
			if($result['resolution']=='') continue;
			$totalCount=$totalCount+intval($result['count']);
		}
		if(!$totalCount) return array();
		foreach($results as $result){
			if($result['resolution']=='') continue; 
			$dimensions=explode('x',$result['resolution']);
			$x=intval(trim($dimensions[0]));
			$y=intval(trim($dimensions[1]));
			
			//round to standard sizes - heights
			//maybe just round height to nearest 50?
			if($y>2250 && $y<=2300) $y=2300;
			elseif($y>1150 && $y<=1200) $y=1200;
			elseif($y>1050 && $y<=1080) $y=1080; 
			elseif($y>1010 && $y<=1050) $y=1050;
			elseif($y>864 && $y<=900) $y=900;			
			elseif($y>824 && $y<=864) $y=864;
			elseif($y>768 && $y<=800) $y=800;
			elseif($y>740 && $y<=768) $y=768;
			elseif($y>680 && $y<=720) $y=720;
			elseif($y>600 && $y<=680) $y=650;
			//elseif($y>600 && $y<=610) $y=610; 
			elseif($y>560 && $y<=600) $y=600;
			elseif($y>440 && $y<=480) $y=480;
			
			//widths
			if($x>2000 && $x<=2048) $x=2048; 
			elseif($x>1880 && $x<=1920) $x=1920;
			elseif($x>1640 && $x<=1680) $x=1680;
			elseif($x>1560 && $x<=1600) $x=1600;
			elseif($x>1400 && $x<=1440) $x=1440;
			elseif($x>1280 && $x<=1300) $x=1300;
			elseif($x>1230 && $x<=1280) $x=1280;
			elseif($x>1112 && $x<=1152) $x=1152;
			elseif($x>980 && $x<=1024) $x=1024;
			elseif($y>810 && $y<=854) $y=854;
			elseif($x>760 && $x<=800) $x=800; 
			elseif($y>680 && $y<=720) $y=720; 
			elseif($x>600 && $x<=640) $x=640;	
					
			$result['resolution']=$x.' x '.$y;
			$familyData[$result['resolution']]['resolution']=$result['resolution'];
			$familyData[$result['resolution']]['sortName'] = intval(substr($result['resolution'],0,strpos(strtolower($result['resolution']),'x')-1));
			$familyData[$result['resolution']]['count'] = intval($familyData[$result['resolution']]['count'])+intval($result['count']);
			$percent = floor(intval($familyData[$result['resolution']]['count']) / $totalCount * 100);
			$familyData[$result['resolution']]['usage'] = $percent;
		}   
		usort($familyData, 'sortByTrafficUsage'); 
		$familyData=array_reverse($familyData);
		$familyData=array_slice($familyData,0,15);
		return $familyData;
	} 
	
	public function getVisitPaths(){
		//$sql='SELECT tv.book_id, count(*) as count FROM traffic_visits AS tv  '; 
		//$sql.=' GROUP BY tv.book_id ORDER BY count desc LIMIT 20'; 
		$sql='SELECT tv.* FROM '.ParticleTrafficTracker::$visits_table.' AS tv  ';
		if(intval($this->siteIdNumber)) $sql.=' WHERE tv.book_id='.intval($this->siteIdNumber).'  ';
		$sql.=' ORDER BY created desc LIMIT 12';
		$visits=$this->db->get_array($sql);
		$visitData=array();
		foreach($visits as $visit){
			$sql='SELECT th.* FROM '.ParticleTrafficTracker::$hits_table.' AS th WHERE th.traffic_visits_id='.intval($visit['traffic_visit_id']).' ORDER BY created DESC LIMIT 500';
			$visit['hitData']=$this->db->get_array($sql);
			if(count($visit['hitData'])) $visitData[]=$visit;
		}
		return $visitData;
	}	
	
	//top sites
	public function getTopBooks(){
		//$sql='SELECT tv.book_id, count(*) as count FROM traffic_visits AS tv  '; 
		//$sql.=' GROUP BY tv.book_id ORDER BY count desc LIMIT 20'; 
		$sql='SELECT tv.book_id, count(tv.traffic_visit_id) as count, (SELECT count(th2.traffic_hits_id) FROM '.ParticleTrafficTracker::$hits_table.' AS th2, '.ParticleTrafficTracker::$visits_table.' AS tv2 WHERE tv2.traffic_visit_id=th2.traffic_visits_id AND tv2.book_id=tv.book_id) as h_count '; 
		$sql.=' FROM traffic_visits AS tv GROUP BY tv.book_id ORDER BY count desc LIMIT 30'; 		
		$results=$this->db->get_array($sql);
		return $results;
	}	
	
	public function getTopPages(){
		$sql='SELECT th.page_url, th.page_url_short, th.page_title, tv.book_id, count(*) as count FROM '.ParticleTrafficTracker::$hits_table.' AS th, '.ParticleTrafficTracker::$visits_table.' AS tv WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' tv.book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' th.page_url_short!="" AND tv.traffic_visit_id=th.traffic_visits_id  GROUP BY th.page_url_short, th.page_title ORDER BY count desc LIMIT 12'; 
		$results=$this->db->get_array($sql);
		return $results;
	}
	
	public function getRecentPages(){
		$sql='SELECT th.page_url, page_url_short, page_title, th.created, tv.book_id, tv.* FROM '.ParticleTrafficTracker::$hits_table.' AS th, '.ParticleTrafficTracker::$visits_table.' AS tv WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' tv.book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' th.page_url_short!="" AND tv.traffic_visit_id=th.traffic_visits_id  ORDER BY th.created desc LIMIT 12'; 
		$results=$this->db->get_array($sql);
		return $results;
	}	
	
	public function getTopReferrers(){
		$sql='SELECT referrer_url, count(*) as count FROM '.ParticleTrafficTracker::$visits_table.' WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' referrer_url!="" GROUP BY referrer_url ORDER BY count desc LIMIT 12'; 
		$results=$this->db->get_array($sql);
		return $results;
	}
	
	public function getRecentReferrers(){
		$sql='SELECT referrer_url, created FROM '.ParticleTrafficTracker::$visits_table.' WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' referrer_url!="" ORDER BY created desc LIMIT 12'; 
		$results=$this->db->get_array($sql);
		return $results;
	}	
	
	public function getTopSearches(){
		$sql='SELECT search_terms, count(*) as count FROM '.ParticleTrafficTracker::$visits_table.' WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' search_terms!="" GROUP BY search_terms ORDER BY count desc LIMIT 12'; 
		$results=$this->db->get_array($sql);
		return $results;
	}
	
	public function getRecentSearches(){
		$sql='SELECT search_terms, created FROM '.ParticleTrafficTracker::$visits_table.' WHERE ';
		if(intval($this->siteIdNumber)) $sql.=' book_id='.intval($this->siteIdNumber).' AND ';
		$sql.=' search_terms!="" ORDER BY created desc LIMIT 12'; 
		$results=$this->db->get_array($sql);
		return $results;
	}
	
	// XML Chart Data	
	//**************************************************************************************************************************
	
	public function getGraphXMLPath($timeframe='week'){
		//get path 
		$path=ParticleTraffic_CACHE_DIR; //ParticleTrafficTools::buildIdSegmentedDirPath(intval($this->siteIdNumber),ParticleTraffic_CACHE_DIR.'traffic_graph_xml/books/');
		/*
		if( !is_writable(ParticleTraffic_CACHE_DIR) )
			throw new Exception("The ParticleTraffic cache directory is not writable.  Make sure this folder has the correct permissions for writing and reading:\n".ParticleTraffic_CACHE_DIR);
		if( !file_exists($path) ){ 
			ParticleTrafficTools::createPath($path); 
			if( !file_exists($path) ) 
				throw new Exception("ParticleTraffic could not create the path to cache your graph xml.  Make sure your cache folder has the correct permissions for writing and reading:\n".$path);
		}
		*/
		if( !is_writable($path) )
			throw new Exception("The ParticleTraffic cache directory is not writable.  Make sure this folder has the correct permissions for writing and reading:\n".$path);
		if( !is_readable($path) )
			throw new Exception("The ParticleTraffic cache directory is not read.  Make sure this folder has the correct permissions for writing and reading:\n".$path);
		
		if($timeframe=='year') $file_name = $path.intval($this->siteIdNumber).'_year.xml';	
		elseif($timeframe=='day') $file_name = $path.intval($this->siteIdNumber).'_day.xml';
		elseif($timeframe=='month') $file_name = $path.intval($this->siteIdNumber).'_month.xml';	
		else $file_name = $path.intval($this->siteIdNumber).'_week.xml';	
		return $file_name;
	}
	
	public function generateGraphXML($visitData,$timeframe='week'){ 
		$xml= "<chart> \r\n";
		$xml.= "\t <chart_type>Line</chart_type> \r\n";
		// $xml.= "\t <tooltip color='FFFFFF' alpha='75' size='11' background_color='8888ff' background_alpha='90' shadow='low' /> \r\n";
		$xml.= "\t <chart_border top_thickness='1' bottom_thickness='1' left_thickness='1' right_thickness='1' /> \r\n";
		//<chart_grid_h alpha='10' thickness='1' type='solid' />
		$xml.= "\t <chart_grid_v alpha='10' thickness='1' type='solid' /> \r\n";
		$xml.= "\t <series_color> \r\n";
		$xml.= "\t\t <color>".str_replace('#','',ParticleTraffic_GRAPH_PAGE_VIEWS_COLOR)."</color> \r\n";
		$xml.= "\t\t <color>".str_replace('#','',ParticleTraffic_GRAPH_VISITS_COLOR)."</color> \r\n";
		$xml.= "\t </series_color> \r\n";
		$xml.= "\t <chart_rect x='30' y='40' width='350' height='200' />";
		$xml.= "\t <legend  x='50' y='10' width='350' height='30' />"; 
		$xml.= "\t <chart_data> \r\n"; 
		
		switch ($timeframe){
			case 'day': 
				$xml.= "\t\t <row> \r\n";
				$xml.= "\t\t\t <null/> \r\n"; 
				for($i=24;$i>=0;$i--){  
					if($i%2==0) $xml.= "\t\t\t <string>".date('g',time()-($i*3600))."</string> \r\n";
					else $xml.= "\t\t\t <string> </string> \r\n"; 
				}
				$xml.= "\t\t </row> \r\n"; 
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Page Views</string> \r\n"; 
				for($i=24;$i>=0;$i--){ 
					$pv=intval( $visitData['hours'][date('Y_n_z_G',time()-($i*3600))]['pageviews'] );
					$xml.= "\t\t\t <number tooltip='".$pv." page views'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n"; 	
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Visits</string> \r\n"; 
				for($i=24;$i>=0;$i--){ 		
					$pv=intval( $visitData['hours'][date('Y_n_z_G',time()-($i*3600))]['visits'] );
					$xml.= "\t\t\t <number tooltip='".$pv." visits'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n";				
				break;
			/*
			case 'month': 
				$xml.= "\t\t <row> \r\n";
				$xml.= "\t\t\t <null/> \r\n"; 
				for($i=24;$i>=0;$i--){  
					if($i%2==0) $xml.= "\t\t\t <string>".date('g',time()-($i*3600))."</string> \r\n";
					else $xml.= "\t\t\t <string> </string> \r\n"; 
				}
				$xml.= "\t\t </row> \r\n"; 
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Page Views</string> \r\n"; 
				for($i=24;$i>=0;$i--){ 
					$pv=intval( $visitData['hours'][date('Y_n_z_G',time()-($i*3600))]['pageviews'] );
					$xml.= "\t\t\t <number tooltip='".$pv." page views'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n"; 	
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Visits</string> \r\n"; 
				for($i=24;$i>=0;$i--){ 		
					$pv=intval( $visitData['hours'][date('Y_n_z_G',time()-($i*3600))]['visits'] );
					$xml.= "\t\t\t <number tooltip='".$pv." visits'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n";				
				break;						
			*/
			case 'year': 
				$xml.= "\t\t <row> \r\n";
				$xml.= "\t\t\t <null/> \r\n"; 
				for($i=12;$i>=0;$i--){ 
					$time=mktime(1,1,1,date('n')-$i,1,date('Y'));
					$xml.= "\t\t\t <string>".date('M',$time)."</string> \r\n"; 
				}
				$xml.= "\t\t </row> \r\n"; 
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Page Views</string> \r\n"; 
				for($i=12;$i>=0;$i--){			
					$time=mktime(1,1,1,date('n')-$i,1,date('Y'));		
					$pv=intval( $visitData['months'][date('Y_n',$time)]['pageviews'] );
					$xml.= "\t\t\t <number tooltip='".$pv." page views'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n"; 	
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Visits</string> \r\n"; 
				for($i=12;$i>=0;$i--){			
					$time=mktime(1,1,1,date('n')-$i,1,date('Y'));	
					$pv=intval( $visitData['months'][date('Y_n',$time)]['visits'] );	
					$xml.= "\t\t\t <number tooltip='".$pv." visits'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n";				
				break;		
			case 'week':
			default:
				$xml.= "\t\t <row> \r\n";
				$xml.= "\t\t\t <null/> \r\n"; 
				for($i=7;$i>=0;$i--){ 
					$xml.= "\t\t\t <string>".date('D',time()-($i*3600*24))."</string> \r\n"; 
				}
				$xml.= "\t\t </row> \r\n"; 
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Page Views</string> \r\n"; 
				for($i=7;$i>=0;$i--){				
					$pv=intval( $visitData['days'][date('Y_n_z',time()-($i*3600*24))]['pageviews'] );	
					$xml.= "\t\t\t <number tooltip='".$pv." page views'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n"; 	
				$xml.= "\t\t <row> \r\n"; 
				$xml.= "\t\t\t <string>Visits</string> \r\n"; 
				for($i=7;$i>=0;$i--){					
					$pv=intval( $visitData['days'][date('Y_n_z',time()-($i*3600*24))]['visits'] );	
					$xml.= "\t\t\t <number tooltip='".$pv." visits'>".$pv."</number> \r\n";  
				}
				$xml.= "\t\t </row> \r\n";				
				break;
		} 
		
		$xml.= "\t </chart_data> \r\n";
		$xml.= "</chart> \r\n";
		
		$file_name = $this->getGraphXMLPath($timeframe);
		
		$Handle = fopen($file_name, 'w');
		fwrite($Handle, $xml);
		fclose($Handle);
	}

}


function sortByTrafficUsage($x, $y){ 
	if ( $x['usage'] == $y['usage'] ) return 0;
	else if ( $x['usage'] < $y['usage'] ) return -1;
	else return 1;
}

?>